package ch.zhaw.gartenverwaltung.backgroundtasks;

import ch.zhaw.gartenverwaltung.CropDetailController;
import ch.zhaw.gartenverwaltung.backgroundtasks.weather.WeatherGradenTaskPlanner;
import ch.zhaw.gartenverwaltung.io.CropList;
import ch.zhaw.gartenverwaltung.io.HardinessZoneNotSetException;
import ch.zhaw.gartenverwaltung.io.PlantList;
import ch.zhaw.gartenverwaltung.io.TaskList;
import ch.zhaw.gartenverwaltung.models.PlantNotFoundException;
import ch.zhaw.gartenverwaltung.types.Task;

import javax.mail.MessagingException;
import java.io.IOException;
import java.time.LocalDate;
import java.util.List;
import java.util.TimerTask;
import java.util.logging.Level;
import java.util.logging.Logger;

public class BackgroundTasks extends TimerTask {
    private static final Logger LOG = Logger.getLogger(CropDetailController.class.getName());
    private final TaskList taskList;
    private final Notifier notifier;
    private final WeatherGradenTaskPlanner weatherGardenTaskPlaner;

    private void movePastTasks() throws IOException {
        List<Task> taskList = this.taskList.getTaskList(LocalDate.MIN, LocalDate.now().minusDays(1));
        taskList.forEach(task -> {
            if (!task.isDone()) {
                task.setNextExecution(LocalDate.now());
            }
        });
    }

    public BackgroundTasks(TaskList taskList, CropList cropList, PlantList plantList) {
        this.taskList = taskList;
        notifier = new Notifier(taskList, plantList, cropList);
        weatherGardenTaskPlaner = new WeatherGradenTaskPlanner(taskList, plantList, cropList);
    }

    @Override
    public void run() {
        try {
            movePastTasks();
        } catch (IOException e) {
            e.printStackTrace();
            LOG.log(Level.SEVERE, "Could not execute Background Task: move past Tasks ", e.getCause());
        }
        try {
            weatherGardenTaskPlaner.refreshTasks();
        } catch (IOException | HardinessZoneNotSetException | PlantNotFoundException e) {
            e.printStackTrace();
            LOG.log(Level.SEVERE, "Could not execute Background Task: Refresh Tasks by WeatherGardenTaskPlaner ", e.getCause());
        }
        try {
            notifier.sendNotifications();
        } catch (IOException | MessagingException | HardinessZoneNotSetException e) {
            e.printStackTrace();
            LOG.log(Level.SEVERE, "Could not execute Background Task: send Notification for due Tasks", e.getCause());
        }
    }
}

